from ..exceptions import NotificationError
from ..utils import log, request


class BaseNotifier(object):
    app = 'Genshin Impact Helper'
    status = ': Test'
    desp = '💌 Send from genshin-impact-helper project.'

    def __init__(self):
        self.name = None
        self.token = None
        self.retcode_key = None
        self.retcode_value = None

    def send(self):
        ...

    def push(self,
             method,
             url,
             params=None,
             data=None,
             json=None,
             headers=None):
        """
        🚫: disabled
        🥳: success
        😳: failure
        """
        if not self.token:
            # log.info(f'{self.name} 🚫')
            return
        try:
            response = request(method, url, 2, params, data, json, headers).json()
        except Exception as e:
            raise NotificationError(f'{self.name} 😳\n{e}')
        else:
            retcode = response.get('data', {}).get(
                self.retcode_key,
                -1) if self.name == 'Server Chan Turbo' else response.get(
                self.retcode_key, -1)
            if retcode == self.retcode_value:
                log.info(f'{self.name} 🥳')

            # Telegram Bot
            elif self.name == 'Telegram Bot' and retcode:
                log.info(f'{self.name} 🥳')
            elif self.name == 'Telegram Bot' and response[self.retcode_value] == 400:
                raise NotificationError(f'{self.name} 😳\n请主动给 bot 发送一条消息并检查 TG_USER_ID 是否正确')
            elif self.name == 'Telegram Bot' and response[self.retcode_value] == 401:
                raise NotificationError(f'{self.name} 😳\nTG_BOT_TOKEN 错误')
            else:
                raise NotificationError(f'{self.name} 😳\n{response}')



from . import (
    bark,
    coolpush,
    customnotifier,
    dingtalkbot,
    discordwebhook,
    igot,
    pushplus,
    serverchan,
    serverchanturbo,
    telegrambot,
    wechatworkapp,
    wechatworkbot, )

from ..exceptions import NoSuchNotifierError

_all_notifiers = {
    'bark': bark.Bark,
    'coolpush': coolpush.CoolPush,
    'custom': customnotifier.CustomNotifier,
    'dingtalkbot': dingtalkbot.DingTalkBot,
    'discordwebhook': discordwebhook.Discord,
    'igot': igot.Igot,
    'pushplus': pushplus.PushPlus,
    'serverchan': serverchan.ServerChan,
    'serverchanturbo': serverchanturbo.ServerChanTurbo,
    'telegrambot': telegrambot.TelegramBot,
    'wechatworkapp': wechatworkapp.WechatWorkApp,
    'wechatworkbot': wechatworkbot.WechatWorkBot,
}


def get_notifiers():
    return list(enumerate(_all_notifiers, start=1))


def get_notifier(name=None):
    notifiers = get_notifiers()
    notifier = [x for x in notifiers if name in x]
    if not notifier:
        raise NoSuchNotifierError(f'No {name} Notifier')
    return _all_notifiers[notifier[0][1]]()


def send2all(text='Genshin Impact Helper',
             status=': Test',
             desp='💌 Send from genshin-impact-helper project.'):
    for notifier in _all_notifiers:
        get_notifier(notifier).send(text, status, desp)



# In order to load the package successfully,
# if the discord_webhook package import fails, it will not be imported.
try:
    from discord_webhook import DiscordWebhook, DiscordEmbed
except Exception as e:
    pass

from .basenotifier import BaseNotifier as Base
from ..config import config
from ..utils import log


class Discord(object):
    @staticmethod
    def send(text=Base.app, status=Base.status, desp=Base.desp):
        if not config.DISCORD_WEBHOOK:
            # log.info(f'Discord 🚫')
            return

        webhook = DiscordWebhook(url=config.DISCORD_WEBHOOK)
        embed = DiscordEmbed(
            title=f'{text} {status}', description=desp, color='03b2f8')
        webhook.add_embed(embed)
        response = webhook.execute()
        if response.status_code == 200:
            log.info(f'Discord 🥳')
        else:
            log.error(f'Discord 😳\n{response}')
